@@ -1,11 +1,11 @@
 /*
  *  File:         debuglogger.h
  *
- *  Author:			  Lukas Zeller (luz@synthesis.ch)
+ *  Author:			  Lukas Zeller (luz@plan44.ch)
  *
  *  Global debug mechanisms
  *
- *  Copyright (c) 2005-2009 by Synthesis AG (www.synthesis.ch)
+ *  Copyright (c) 2005-2011 by Synthesis AG + plan44.ch
  *
  *  2005-08-04 : luz : created
  *
@@ -58,11 +58,22 @@ typedef enum {
 } TDbgSubthreadModes;
 
 
+/// @brief HTML linking into source code
+typedef enum {
+  dbgsource_none,     ///< do not include links into source code in HTML logs
+  dbgsource_hint,     ///< no links, but info about what file/line number the message comes from 
+  dbgsource_doxygen,  ///< include link into doxygen prepared HTML version of source code
+  dbgsource_txmt,     ///< include txmt:// link (understood by TextMate and BBEdit) into source code
+  numDbgSourceModes
+} TDbgSourceModes;
+
+
 #ifndef HARDCODED_CONFIG
 extern cAppCharP const DbgOutFormatNames[numDbgOutFormats];
 extern cAppCharP const DbgFoldingModeNames[numDbgFoldingModes];
 extern cAppCharP const DbgFlushModeNames[numDbgFlushModes];
 extern cAppCharP const DbgSubthreadModeNames[numDbgSubthreadModes];
+extern cAppCharP const DbgSourceModeNames[numDbgSourceModes];
 #endif
 extern cAppCharP const DbgOutFormatExtensions[numDbgOutFormats];
 
@@ -78,12 +89,17 @@ public:
   string fIndentString; ///< indent string
   string fCustomPrefix; ///< custom prefix (different xml header or html with different styles for example)
   string fCustomSuffix; ///< custom suffix (should match prefix)
+  string fBasename; ///< the initial part of the log file name, can override the hard-coded TARGETID (empty if unset)
   bool fSeparateMsgs; ///< separate message lines (needed especially in XML to avoid unformatted PCDATA block)
   bool fTimestampStructure; ///< include timestamp for structure elements (blocks)
   bool fTimestampForAll; ///< include timestamp information for every message
   bool fThreadIDForAll; ///< include thread ID information for every message
   TDbgFlushModes fFlushMode; ///< how and when to flush
   TDbgFoldingModes fFoldingMode; ///< if and how to fold HTML output
+  #ifdef SYDEBUG_LOCATION
+  TDbgSourceModes fSourceLinkMode; ///< if and how to link with source code
+  string fSourceRootPath; ///< defines root path for source links
+  #endif
   bool fAppend; ///< if set, existing debug files will not be overwritten, but appended to
   TDbgSubthreadModes fSubThreadMode; ///< how to handle debug messages from subthreads
   uInt32 fSubThreadBufferMax; ///< how much to buffer for subthread maximally
@@ -236,18 +252,18 @@ public:
   /// @param aDbgMask debug mask, bits set here must be set in the debuglogger's own mask in order to display the debug text
   /// @param aText[in] text to be written out
   /// @param aTextSize[in] if>0, this is the maximum number of chars to output from aText
-  virtual void DebugPuts(uInt32 aDbgMask, cAppCharP aText, stringSize aTextSize=0, bool aPreFormatted=false);
+  virtual void DebugPuts(TDBG_LOCATION_PROTO uInt32 aDbgMask, cAppCharP aText, stringSize aTextSize=0, bool aPreFormatted=false);
   /// @brief Write formatted text to debug output channel.
   /// @param aDbgMask debug mask, bits set here must be set in the debuglogger's own mask in order to display the debug text
   /// @param aFormat[in] format text in vprintf style to be written out
   /// @param aArgs[in] varargs in vprintf style
-  virtual void DebugVPrintf(uInt32 aDbgMask, cAppCharP aFormat, va_list aArgs);
+  virtual void DebugVPrintf(TDBG_LOCATION_PROTO uInt32 aDbgMask, cAppCharP aFormat, va_list aArgs);
   /// @brief Write formatted text to debug output channel.
   /// @param aDbgMask debug mask, bits set here must be set in the debuglogger's own mask in order to display the debug text
   /// @param aFormat[in] format text in printf style to be written out
-  void DebugPrintf(uInt32 aDbgMask, cAppCharP aFormat, ...)
+  void DebugPrintf(TDBG_LOCATION_PROTO uInt32 aDbgMask, cAppCharP aFormat, ...)
   	#ifdef __GNUC__
-    __attribute__((format(printf, 3, 4)))
+    __attribute__((format(printf, TDBG_LOCATION_ARG_NUM + 3, TDBG_LOCATION_ARG_NUM + 4)))
 		#endif
     ;
   /// @brief set debug mask to be used for next DebugPrintfLastMask() call
@@ -255,9 +271,9 @@ public:
   virtual TDebugLoggerBase &setNextMask(uInt32 aDbgMask);
   /// @brief like DebugPrintf(), but using mask previously set by setNextMask()
   /// @param aFormat[in] format text in printf style to be written out
-  void DebugPrintfLastMask(cAppCharP aFormat, ...)
+  void DebugPrintfLastMask(TDBG_LOCATION_PROTO cAppCharP aFormat, ...)
 		#ifdef __GNUC__
-    __attribute__((format(printf, 2, 3)))
+    __attribute__((format(printf, TDBG_LOCATION_ARG_NUM + 2, TDBG_LOCATION_ARG_NUM + 3)))
 		#endif
     ;
   // - Blocks
@@ -268,33 +284,33 @@ public:
   /// @param aBlockFmt[in]   Format string for additional Block info. Should contain one or multiple tag=value pairs, separated by the pipe char |.
   ///                          This will be used to generate XML attributes or other identifiers.
   /// @param aArgs[in]         varargs in vprintf style for aBlockFmt
-  virtual void DebugVOpenBlock(cAppCharP aBlockName, cAppCharP aBlockTitle, bool aCollapsed, cAppCharP aBlockFmt, va_list aArgs);
+  virtual void DebugVOpenBlock(TDBG_LOCATION_PROTO cAppCharP aBlockName, cAppCharP aBlockTitle, bool aCollapsed, cAppCharP aBlockFmt, va_list aArgs);
   /// @brief Open structure Block, printf style variant
-  void DebugOpenBlock(cAppCharP aBlockName, cAppCharP aBlockTitle, bool aCollapsed, cAppCharP aBlockFmt, ...)
+  void DebugOpenBlock(TDBG_LOCATION_PROTO cAppCharP aBlockName, cAppCharP aBlockTitle, bool aCollapsed, cAppCharP aBlockFmt, ...)
     #ifdef __GNUC__
-    __attribute__((format(printf, 5, 6)))
+    __attribute__((format(printf, TDBG_LOCATION_ARG_NUM + 5, TDBG_LOCATION_ARG_NUM + 6)))
 		#endif
       ;
-  void DebugOpenBlockExpanded(cAppCharP aBlockName, cAppCharP aBlockTitle, cAppCharP aBlockFmt, ...)
+  void DebugOpenBlockExpanded(TDBG_LOCATION_PROTO cAppCharP aBlockName, cAppCharP aBlockTitle, cAppCharP aBlockFmt, ...)
     #ifdef __GNUC__
-    __attribute__((format(printf, 4, 5)))
+    __attribute__((format(printf, TDBG_LOCATION_ARG_NUM + 4, TDBG_LOCATION_ARG_NUM + 5)))
 		#endif
     ;
-  void DebugOpenBlockCollapsed(cAppCharP aBlockName, cAppCharP aBlockTitle, cAppCharP aBlockFmt, ...)
+  void DebugOpenBlockCollapsed(TDBG_LOCATION_PROTO cAppCharP aBlockName, cAppCharP aBlockTitle, cAppCharP aBlockFmt, ...)
     #ifdef __GNUC__
-    __attribute__((format(printf, 4, 5)))
+    __attribute__((format(printf, TDBG_LOCATION_ARG_NUM + 4, TDBG_LOCATION_ARG_NUM + 5)))
 		#endif
     ;
   /// @brief Open structure Block, without any attributes
-  virtual void DebugOpenBlock(cAppCharP aBlockName, cAppCharP aBlockTitle=NULL, bool aCollapsed=false);
-  /// @brief Close structure Block. Name is used to close eventually unclosed contained Blocks automatically.
-  virtual void DebugCloseBlock(cAppCharP aBlockName);
+  virtual void DebugOpenBlock(TDBG_LOCATION_PROTO cAppCharP aBlockName, cAppCharP aBlockTitle=NULL, bool aCollapsed=false);
+  /// @brief Close structure Block. Name is used to close possibly unclosed contained Blocks automatically.
+  virtual void DebugCloseBlock(TDBG_LOCATION_PROTO cAppCharP aBlockName);
 protected:
   // helper methods
   /// @brief start debugging output if needed and sets fOutStarted
   bool DebugStartOutput(void);
   /// @brief Output single line to debug channel (includes indenting, but no other formatting)
-  void DebugPutLine(cAppCharP aText, stringSize aTextSize=0, bool aPre=false);
+  void DebugPutLine(TDBG_LOCATION_PROTO cAppCharP aText, stringSize aTextSize=0, bool aPre=false);
   /// @brief finalize debugging output
   void DebugFinalizeOutput(void);
   /// @brief get block number
@@ -304,7 +320,11 @@ protected:
   /// @brief internal helper for closing debug Blocks
   /// @param aBlockName[in]   Name of Block to close. All Blocks including the first with given name will be closed. If NULL, all Blocks will be closed.
   /// @param aCloseComment[in]  Comment about closing Block. If NULL, no comment will be shown (unless implicit closes occur, which auto-creates a comment)
-  void internalCloseBlocks(cAppCharP aBlockName, cAppCharP aCloseComment);
+  void internalCloseBlocks(TDBG_LOCATION_PROTO cAppCharP aBlockName, cAppCharP aCloseComment);
+	#ifdef SYDEBUG_LOCATION
+	/// @brief turn text into link to source code
+	string dbg2Link(const TDbgLocation &aTDbgLoc, const string &aTxt);
+  #endif // SYDEBUG_LOCATION
   // Variables
   TDbgOut *fDbgOutP; // the debug output
   string fDbgPath; // the output path+filename (w/o extension)
@@ -341,10 +361,10 @@ public:
   virtual ~TDebugLogger();
   // methods
   #ifdef MULTI_THREAD_SUPPORT
-  virtual void DebugPuts(uInt32 aDbgMask, cAppCharP aText, stringSize aTextSize=0, bool aPreFormatted=false);
-  virtual void DebugVPrintf(uInt32 aDbgMask, cAppCharP aFormat, va_list aArgs);
-  virtual void DebugVOpenBlock(cAppCharP aBlockName, cAppCharP aBlockTitle, bool aCollapsed, cAppCharP aBlockFmt, va_list aArgs);
-  virtual void DebugCloseBlock(cAppCharP aBlockName);
+  virtual void DebugPuts(TDBG_LOCATION_PROTO uInt32 aDbgMask, cAppCharP aText, stringSize aTextSize=0, bool aPreFormatted=false);
+  virtual void DebugVPrintf(TDBG_LOCATION_PROTO uInt32 aDbgMask, cAppCharP aFormat, va_list aArgs);
+  virtual void DebugVOpenBlock(TDBG_LOCATION_PROTO cAppCharP aBlockName, cAppCharP aBlockTitle, bool aCollapsed, cAppCharP aBlockFmt, va_list aArgs);
+  virtual void DebugCloseBlock(TDBG_LOCATION_PROTO cAppCharP aBlockName);
   virtual TDebugLoggerBase &setNextMask(uInt32 aDbgMask);
   #endif
   // - thread debug output serializing
@@ -357,7 +377,7 @@ public:
   ///   to write into the output channel until a new main thread gains control via DebugDefineMainThread();
   void DebugThreadOutputDone(bool aRemoveIt=false);
   /// @brief Define the current calling thread as the main debug thread
-  /// Note: This is used for example when starting to process the next request of a session which eventually
+  /// Note: This is used for example when starting to process the next request of a session which possibly
   //        occurs from another thread).
   void DebugDefineMainThread(void);
 private:
